;**************************************************************************
;*
;* FILE  screen_io.asm
;* Copyright (c) 2005, 2006 Daniel Kahlin <daniel@kahlin.net>
;* Written by Daniel Kahlin <daniel@kahlin.net>
;* $Id: screen_io.asm,v 1.10 2007-06-03 19:01:17 tlr Exp $
;*
;* DESCRIPTION
;*   screen and keyboard input and output functions.
;*
;******

	
	seg	code
;**************************************************************************
;*
;* NAME  ask_sure
;*
;* DESCRIPTION
;*   Ask "are you sure?".   C=1 if yes.
;*   
;******
ask_sure:	
	lda	#<sure_msg
	ldy	#>sure_msg
	jsr	print
as_lp1:
	jsr	$ffe4
	beq	as_lp1
	cmp	#"Y"
	beq	as_ex1
	clc
	rts
as_ex1:
	sec
	rts

sure_msg:
	dc.b	"ARE YOU SURE (Y/N)?",13,0

;**************************************************************************
;*
;* NAME  print
;*
;* DESCRIPTION
;*   output string
;*   
;******
print:
	sta	ptr_zp
	sty	ptr_zp+1
	ldy	#0
prt_lp1:
	lda	(ptr_zp),y
	beq	prt_ex1
	jsr	$ffd2
	iny
	bne	prt_lp1
	inc	ptr_zp+1
	jmp	prt_lp1
prt_ex1:
	rts


;**************************************************************************
;*
;* NAME  print_space, print_cr
;*
;* DESCRIPTION
;*   print routines for common chars.
;*   
;******
print_space:
	lda	#" "
	dc.b	$2c
print_cr:
	lda	#13
	jmp	$ffd2

;**************************************************************************
;*
;* NAME  clear_upto
;*
;* DESCRIPTION
;*   Clear chars upto and including column Y
;*   
;******
clear_upto:
	lda	#" "
cn_lp1:
	cpy	$d3
	bcc	cu_ex1
	sta	($d1),y
	dey
	bpl	cn_lp1
cu_ex1:
	rts


;**************************************************************************
;*
;* NAME  print_hex
;*
;* DESCRIPTION
;*   output hex byte.
;*   
;******
print_hex:
	pha
	lsr
	lsr
	lsr
	lsr
	jsr	ph_skp1
	pla
	and	#$0f
ph_skp1:
	clc
	adc	#"0"
	cmp	#"9"+1
	bcc	ph_skp2
	adc	#"A"-("9"+1)-1
ph_skp2:
	jmp	$ffd2


;**************************************************************************
;*
;* NAME  input_filename, input_filename_pre
;*
;* DESCRIPTION
;*   input filename and set up.
;*
;******
input_filename_pre:
	ldx	$d3
	jsr	print
	stx	$d3
input_filename:
	ldy	#0
if_lp1:
	jsr	$ffcf
	cmp	#13
	beq	if_ex1
	sta	filename,y
	iny
	cpy	#filename_end-filename
	bne	if_lp1
if_ex1:
	tya
	ldx	#<filename
	ldy	#>filename
	jsr	$ffbd
	rts

	seg.u	bss
filename:
	ds.b	40
filename_end:
	
	seg	code
;**************************************************************************
;*
;* NAME  input_y_n_pre
;*
;* DESCRIPTION
;*   input y/n.
;*   Acc="Y" or "N", C=1 if error
;*   
;******
input_y_n_pre:
	ldx	$d3
	jsr	$ffd2
	stx	$d3
	stx	xtmp_zp
	jsr	$ffcf
	cmp	#"Y"
	beq	iyn_ex1	
	cmp	#"N"
	bne	iyn_fl1
iyn_ex1:
	clc
	rts
iyn_fl1:
	ldx	xtmp_zp
	stx	$d3	
	lda	#"?"
	jsr	$ffd2
	sec
	rts

;**************************************************************************
;*
;* NAME  input_byte
;*
;* DESCRIPTION
;*   input hex byte.
;*   Acc=byte, C=1 if error
;*   
;******
input_byte_pre:
	ldx	$d3
	jsr	print_hex
	stx	$d3
input_byte:
	ldx	$d3
	stx	xtmp_zp
	jsr	input_hex
	bcs	ib_fl1
	ldx	xtmp_zp
	stx	$d3	
	pha
	jsr	print_hex
	pla
	clc
	rts
ib_fl1:
	ldx	xtmp_zp
	stx	$d3	
	lda	#"?"
	jsr	$ffd2
	sec
	rts

;**************************************************************************
;*
;* NAME  input_word
;*
;* DESCRIPTION
;*   input hex word.
;*   Acc=LSB, Y=MSB,  C=1 if error
;*   
;******
input_word_pre:
	sta	tmpptr_zp
	sty	tmpptr_zp+1
	ldx	$d3
	jsr	iw_print
	stx	$d3
input_word:
	ldx	$d3
	stx	xtmp_zp
	jsr	input_hex
	bcs	iw_fl1
	sta	tmpptr_zp+1
	jsr	input_hex
	bcs	iw_fl1
	sta	tmpptr_zp
	ldx	xtmp_zp
	stx	$d3
	
	jsr	iw_print

	lda	tmpptr_zp
	ldy	tmpptr_zp+1
	clc
	rts
iw_fl1:
	ldx	xtmp_zp
	stx	$d3	
	lda	#"?"
	jsr	$ffd2
	stx	$d3	
	sec
	rts

;**************************************************************************
;*
;* NAME  input_triplet
;*
;* DESCRIPTION
;*   input hex triplet.
;*   Acc=LSB, Y=MidSB, X=MSB,  C=1 if error
;*   
;******
input_triplet_pre:
	sta	tmpptr_zp
	sty	tmpptr_zp+1
	stx	tmpptr_zp+2
	ldx	$d3
	jsr	it_print
	stx	$d3
input_triplet:
	ldx	$d3
	stx	xtmp_zp
	jsr	input_hex
	bcs	it_fl1
	sta	tmpptr_zp+2
	jsr	input_hex
	bcs	it_fl1
	sta	tmpptr_zp+1
	jsr	input_hex
	bcs	it_fl1
	sta	tmpptr_zp
	ldx	xtmp_zp
	stx	$d3	

	jsr	it_print

	lda	tmpptr_zp
	ldy	tmpptr_zp+1
	ldx	tmpptr_zp+2
	clc
	rts
it_fl1:
	ldx	xtmp_zp
	stx	$d3	
	lda	#"?"
	jsr	$ffd2
	stx	$d3	
	sec
	rts

it_print:
	lda	tmpptr_zp+2
	jsr	print_hex
iw_print:
	lda	tmpptr_zp+1
	jsr	print_hex
	lda	tmpptr_zp
	jmp	print_hex
	
			
;**************************************************************************
;*
;* NAME  input_hex
;*
;* DESCRIPTION
;*   internal input hex byte routine.
;*   Acc=byte, C=1 if error
;*   
;******
input_hex:
	ldx	#2
ih_lp1:
	jsr	$ffcf
	cmp	#13
	beq	ih_fl1

	ldy	#16-1
ih_lp2:
	cmp	hextab,y
	beq	ih_skp1
	dey
	bpl	ih_lp2
; y=$ff
ih_fl1:
	sec
	rts
ih_skp1:
	asl	tmp_zp
	asl	tmp_zp
	asl	tmp_zp
	asl	tmp_zp
	tya
	ora	tmp_zp
	sta	tmp_zp
	dex
	bne	ih_lp1
ih_ex1:
	lda	tmp_zp
	clc
	rts

hextab:
	dc.b	"0123456789ABCDEF"	

	if	DEBUG_OUT
;**************************************************************************
;*
;* NAME  dbg_print
;*
;* DESCRIPTION
;*   Print, preserving status reg.
;*   
;******
dbg_print:
	php
	jsr	print
	plp
	rts

;**************************************************************************
;*
;* NAME  dbg_printhex
;*
;* DESCRIPTION
;*   Print hex byte from Acc at column Y.
;*   
;******
dbg_printhex:
	stx	dbg_xtmp
	pha
	lsr
	lsr
	lsr
	lsr
	jsr	dbph_skp1
	pla
dbph_skp1:
	and	#$0f
	tax
	lda	dbg_hextab,x
	sta	($d1),y
	iny
	ldx	dbg_xtmp
	rts

dbg_hextab:
	dc.b	"0123456789",1,2,3,4,5,6

dbg_xtmp:
	dc.b	0
dbg_ytmp:
	dc.b	0
	
;**************************************************************************
;*
;* NAME  dbg_printcr
;*
;* DESCRIPTION
;*   Print cr.
;*   
;******
dbg_printcr:
	php
	pha
	lda	#13
	jsr	$ffd2
	pla
	plp
	rts

;**************************************************************************
;*
;* NAME  dbg_printfptr
;*
;* DESCRIPTION
;*   Print the flash pointer.
;*   
;******
dbg_printfptr:
	lda	fptr_zp+2
	jsr	dbg_printhex
	lda	fptr_zp+1
	jsr	dbg_printhex
	rts
dbg_printfptr_full:
	jsr	dbg_printfptr
	lda	fptr_zp
	jmp	dbg_printhex
;**************************************************************************
;*
;* NAME  dbg_printeptr
;*
;* DESCRIPTION
;*   Print the flash pointer.
;*   
;******
dbg_printeptr:
	lda	eptr_zp+2
	jsr	dbg_printhex
	lda	eptr_zp+1
	jsr	dbg_printhex
	rts
dbg_printeptr_full:
	jsr	dbg_printeptr
	lda	eptr_zp
	jmp	dbg_printhex
;**************************************************************************
;*
;* NAME  dbg_printrptr
;*
;* DESCRIPTION
;*   Print the ram pointer.
;*   
;******
dbg_printrptr:
	lda	rptr_zp+2
	sec
	sbc	#BUFFER_START>>16
	jsr	dbg_printhex
	lda	rptr_zp+1
	jsr	dbg_printhex
	rts
dbg_printrptr_full:
	jsr	dbg_printrptr
	lda	rptr_zp
	jmp	dbg_printhex
	
	endif
; eof
